@@ -32,9 +32,14 @@
 #include "context/context.h"
 #include "context/internalization_codes.h"
 #include "model/models.h"
+#include "solvers/bv/dimacs_printer.h"
+#include "solvers/cdcl/delegate.h"
 #include "solvers/funs/fun_solver.h"
 #include "solvers/simplex/simplex.h"
 
+#include "api/yices_globals.h"
+#include "mt/thread_macros.h"
+
 
 
 /*
@@ -49,15 +54,13 @@ static void trace_stats(smt_core_t *core, const char *when, uint32_t level) {
 	       "(%-10s %8"PRIu64" %10"PRIu64" %8"PRIu64" %8"PRIu32" %8"PRIu32" %8"PRIu64" %8"PRIu32" %8"PRIu64" %7.1f)\n",
 	       when, core->stats.conflicts, core->stats.decisions, core->stats.random_decisions,
 	       num_binary_clauses(core), num_prob_clauses(core), num_prob_literals(core),
-	       num_learned_clauses(core), num_learned_literals(core),
-	       (double) num_learned_literals(core)/num_learned_clauses(core));
+	       num_learned_clauses(core), num_learned_literals(core), avg_learned_clause_size(core));
 #if 0
   fprintf(stderr,
 	  "(%-10s %8"PRIu64" %10"PRIu64" %8"PRIu64" %8"PRIu32" %8"PRIu32" %8"PRIu64" %8"PRIu32" %8"PRIu64" %7.1f)\n",
 	  when, core->stats.conflicts, core->stats.decisions, core->stats.random_decisions,
 	  num_binary_clauses(core), num_prob_clauses(core), num_prob_literals(core),
-	  num_learned_clauses(core), num_learned_literals(core),
-	  (double) num_learned_literals(core)/num_learned_clauses(core));
+	  num_learned_clauses(core), num_learned_literals(core), avg_learned_clause_size(core));
 #endif
 }
 
@@ -380,7 +383,7 @@ static void solve(smt_core_t *core, const param_t *params, uint32_t n, const lit
     d_threshold = params->d_threshold;
     // HACK to activate the Luby heuristic:
     // c_factor must be 0.0 and fast_restart must be true
-    luby = params->c_factor == 0.0; 
+    luby = params->c_factor == 0.0;
   }
 
   reduce_threshold = (uint32_t) (num_prob_clauses(core) * params->r_fraction);
@@ -463,7 +466,6 @@ static void solve(smt_core_t *core, const param_t *params, uint32_t n, const lit
 
 /*
  * Initialize the search parameters based on params.
- * If params is NULL, we use default values.
  */
 static void context_set_search_parameters(context_t *ctx, const param_t *params) {
   smt_core_t *core;
@@ -472,10 +474,6 @@ static void context_set_search_parameters(context_t *ctx, const param_t *params)
   fun_solver_t *fsolver;
   uint32_t quota;
 
-  if (params == NULL) {
-    params = get_default_params();
-  }
-
   /*
    * Set core parameters
    */
@@ -549,18 +547,30 @@ static void context_set_search_parameters(context_t *ctx, const param_t *params)
   }
 }
 
+static smt_status_t _o_call_mcsat_solver(context_t *ctx, const param_t *params) {
+  mcsat_solve(ctx->mcsat, params, NULL, 0, NULL);
+  return mcsat_status(ctx->mcsat);
+}
+
+static smt_status_t call_mcsat_solver(context_t *ctx, const param_t *params) {
+  MT_PROTECT(smt_status_t, __yices_globals.lock, _o_call_mcsat_solver(ctx, params));
+}
 
 /*
  * Initialize search parameters then call solve
  * - if ctx->status is not IDLE, return the status.
+ * - if params is NULL, we use default values.
  */
 smt_status_t check_context(context_t *ctx, const param_t *params) {
   smt_core_t *core;
   smt_status_t stat;
 
+  if (params == NULL) {
+    params = get_default_params();
+  }
+
   if (ctx->mcsat != NULL) {
-    mcsat_solve(ctx->mcsat, params);
-    return mcsat_status(ctx->mcsat);
+    return call_mcsat_solver(ctx, params);
   }
 
   core = ctx->core;
@@ -584,12 +594,13 @@ smt_status_t check_context_with_assumptions(context_t *ctx, const param_t *param
   smt_core_t *core;
   smt_status_t stat;
 
-  assert(ctx->mcsat == NULL); // MC-SAT doesn't support assumptions yet
-
   core = ctx->core;
   stat = smt_status(core);
   if (stat == STATUS_IDLE) {
     // clean state
+    if (params == NULL) {
+      params = get_default_params();
+    }
     context_set_search_parameters(ctx, params);
     solve(core, params, n, a);
     stat = smt_status(core);
@@ -598,6 +609,29 @@ smt_status_t check_context_with_assumptions(context_t *ctx, const param_t *param
   return stat;
 }
 
+/*
+ * Check with given model
+ * - if mcsat status is not IDLE, return the status
+ */
+smt_status_t check_context_with_model(context_t *ctx, const param_t *params, model_t* mdl, uint32_t n, const term_t t[]) {
+  smt_status_t stat;
+
+  assert(ctx->mcsat != NULL);
+
+  stat = mcsat_status(ctx->mcsat);
+  if (stat == STATUS_IDLE) {
+    mcsat_solve(ctx->mcsat, params, mdl, n, t);
+    stat = mcsat_status(ctx->mcsat);
+
+    // BD: this looks wrong. We shouldn't call clear yet.
+    // we want the status to remain STATUS_UNSAT until the next call to check or assert.
+    //    if (n > 0 && stat == STATUS_UNSAT && context_supports_multichecks(ctx)) {
+    //      context_clear(ctx);
+    //    }
+  }
+
+  return stat;
+}
 
 
 /*
@@ -645,6 +679,186 @@ smt_status_t precheck_context(context_t *ctx) {
 
 
 
+/*
+ * Solve using another SAT solver
+ * - sat_solver = name of the solver to use
+ * - verbosity = verbosity level (0 means quiet)
+ * - this may be used only for BV or pure SAT problems
+ * - we perform one round of propagation to convert the problem to CNF
+ * - then we call an external SAT solver on the CNF problem
+ */
+smt_status_t check_with_delegate(context_t *ctx, const char *sat_solver, uint32_t verbosity) {
+  smt_status_t stat;
+  smt_core_t *core;
+  delegate_t delegate;
+  bvar_t x;
+  bval_t v;
+
+  core = ctx->core;
+
+  stat = smt_status(core);
+  if (stat == STATUS_IDLE) {
+    start_search(core, 0, NULL);
+    smt_process(core);
+    stat = smt_status(core);
+
+    assert(stat == STATUS_UNSAT || stat == STATUS_SEARCHING ||
+	   stat == STATUS_INTERRUPTED);
+
+    if (stat == STATUS_SEARCHING) {
+      if (smt_easy_sat(core)) {
+	stat = STATUS_SAT;
+      } else {
+	// call the delegate
+	init_delegate(&delegate, sat_solver, num_vars(core));
+	delegate_set_verbosity(&delegate, verbosity);
+
+	stat = solve_with_delegate(&delegate, core);
+	set_smt_status(core, stat);
+	if (stat == STATUS_SAT) {
+	  for (x=0; x<num_vars(core); x++) {
+	    v = delegate_get_value(&delegate, x);
+	    set_bvar_value(core, x, v);
+	  }
+	}
+	delete_delegate(&delegate);
+      }
+    }
+  }
+
+  return stat;
+}
+
+
+/*
+ * Bit-blast then export to DIMACS
+ * - filename = name of the output file
+ * - status = status of the context after bit-blasting
+ *
+ * If ctx status is IDLE
+ * - perform one round of propagation to conver the problem to CNF
+ * - export the CNF to DIMACS
+ *
+ * If ctx status is not IDLE,
+ * - store the stauts in *status and do nothing else
+ *
+ * Return code:
+ *  1 if the DIMACS file was created
+ *  0 if the problem was solved by the propagation round
+ * -1 if there was an error in creating or writing to the file.
+ */
+int32_t bitblast_then_export_to_dimacs(context_t *ctx, const char *filename, smt_status_t *status) {
+  smt_core_t *core;
+  FILE *f;
+  smt_status_t stat;
+  int32_t code;
+
+  core = ctx->core;
+
+  code = 0;
+  stat = smt_status(core);
+  if (stat == STATUS_IDLE) {
+    start_search(core, 0, NULL);
+    smt_process(core);
+    stat = smt_status(core);
+
+    assert(stat == STATUS_UNSAT || stat == STATUS_SEARCHING ||
+	   stat == STATUS_INTERRUPTED);
+
+    if (stat == STATUS_SEARCHING) {
+      code = 1;
+      f = fopen(filename, "w");
+      if (f == NULL) {
+	code = -1;
+      } else {
+	dimacs_print_bvcontext(f, ctx);
+	if (ferror(f)) code = -1;
+	fclose(f);
+      }
+    }
+  }
+
+  *status = stat;
+
+  return code;
+}
+
+
+/*
+ * Simplify then export to Dimacs:
+ * - filename = name of the output file
+ * - status = status of the context after CNF conversion + preprocessing
+ *
+ * If ctx status is IDLE
+ * - perform one round of propagation to convert the problem to CNF
+ * - export the CNF to y2sat for extra preprocessing then export that to DIMACS
+ *
+ * If ctx status is not IDLE, the function stores that in *status
+ * If y2sat preprocessing solves the formula, return the status also in *status
+ *
+ * Return code:
+ *  1 if the DIMACS file was created
+ *  0 if the problems was solved by preprocessing (or if ctx status is not IDLE)
+ * -1 if there was an error creating or writing to the file.
+ */
+int32_t process_then_export_to_dimacs(context_t *ctx, const char *filename, smt_status_t *status) {
+  smt_core_t *core;
+  FILE *f;
+  smt_status_t stat;
+  delegate_t delegate;
+  bvar_t x;
+  bval_t v;
+  int32_t code;
+
+  core = ctx->core;
+
+  code = 0;
+  stat = smt_status(core);
+  if (stat == STATUS_IDLE) {
+    start_search(core, 0, NULL);
+    smt_process(core);
+    stat = smt_status(core);
+
+    assert(stat == STATUS_UNSAT || stat == STATUS_SEARCHING ||
+	   stat == STATUS_INTERRUPTED);
+
+    if (stat == STATUS_SEARCHING) {
+      if (smt_easy_sat(core)) {
+	stat = STATUS_SAT;
+      } else {
+	// call the delegate
+	init_delegate(&delegate, "y2sat", num_vars(core));
+	delegate_set_verbosity(&delegate, 0);
+
+	stat = preprocess_with_delegate(&delegate, core);
+	set_smt_status(core, stat);
+	if (stat == STATUS_SAT) {
+	  for (x=0; x<num_vars(core); x++) {
+	    v = delegate_get_value(&delegate, x);
+	    set_bvar_value(core, x, v);
+	  }
+	} else if (stat == STATUS_UNKNOWN) {
+	  code = 1;
+	  f = fopen(filename, "w");
+	  if (f == NULL) {
+	    code = -1;
+	  } else {
+	    export_to_dimacs_with_delegate(&delegate, f);
+	    if (ferror(f)) code = -1;
+	    fclose(f);
+	  }
+	}
+
+	delete_delegate(&delegate);
+      }
+    }
+  }
+
+  *status = stat;
+
+  return code;
+}
+
 
 
 /*
@@ -831,12 +1045,13 @@ static void build_term_value(context_t *ctx, model_t *model, term_t t) {
 
 
 /*
- * Build a model for the current context
+ * Build a model for the current context (including all satellite solvers)
  * - the context status must be SAT (or UNKNOWN)
  * - if model->has_alias is true, we store the term substitution
  *   defined by ctx->intern_tbl into the model
+ * - cleanup of satellite models needed using clean_solver_models()
  */
-void context_build_model(model_t *model, context_t *ctx) {
+void build_model(model_t *model, context_t *ctx) {
   term_table_t *terms;
   uint32_t i, n;
   term_t t;
@@ -875,14 +1090,17 @@ void context_build_model(model_t *model, context_t *ctx) {
     if (good_term_idx(terms, i)) {
       t = pos_occ(i);
       if (term_kind(terms, t) == UNINTERPRETED_TERM) {
-	build_term_value(ctx, model, t);
+        build_term_value(ctx, model, t);
       }
     }
   }
+}
 
-  /*
-   * Cleanup
-   */
+
+/*
+ * Cleanup solver models
+ */
+void clean_solver_models(context_t *ctx) {
   if (context_has_arith_solver(ctx)) {
     ctx->arith.free_model(ctx->arith_solver);
   }
@@ -892,7 +1110,22 @@ void context_build_model(model_t *model, context_t *ctx) {
   if (context_has_egraph(ctx)) {
     egraph_free_model(ctx->egraph);
   }
+}
 
+
+
+/*
+ * Build a model for the current context
+ * - the context status must be SAT (or UNKNOWN)
+ * - if model->has_alias is true, we store the term substitution
+ *   defined by ctx->intern_tbl into the model
+ */
+void context_build_model(model_t *model, context_t *ctx) {
+  // Build solver models and term values
+  build_model(model, ctx);
+
+  // Cleanup
+  clean_solver_models(ctx);
 }
 
 
@@ -965,3 +1198,13 @@ void context_build_unsat_core(context_t *ctx, ivector_t *v) {
     v->data[i] = t;
   }
 }
+
+
+/*
+ * MODEL INTERPOLANT
+ */
+term_t context_get_unsat_model_interpolant(context_t *ctx) {
+  assert(ctx->mcsat != NULL);
+  return mcsat_get_unsat_model_interpolant(ctx->mcsat);
+}
+
