@@ -38,6 +38,7 @@
 #include "solvers/cdcl/gates_manager.h"
 #include "solvers/cdcl/smt_core.h"
 #include "solvers/egraph/egraph.h"
+#include "terms/bvpoly_buffers.h"
 #include "terms/conditionals.h"
 #include "terms/int_rational_hash_maps.h"
 #include "terms/poly_buffer.h"
@@ -47,6 +48,7 @@
 #include "utils/int_queues.h"
 #include "utils/int_stack.h"
 #include "utils/int_vectors.h"
+#include "utils/object_stack.h"
 #include "utils/mark_vectors.h"
 #include "utils/pair_hash_map2.h"
 
@@ -468,6 +470,9 @@ typedef struct arith_interface_s {
  *    - const->nbits = number of bits
  *    - const->bits = array of uint32_t words (constant value)
  *
+ * 2c) thvar_t create_zero(void *solver, uint32_t n)
+ *     - must create the zero constant of n bits
+ *
  * 3a) thvar_t create_poly(void *solver, bvpoly_t *p, thvar_t *map)
  * 3b) thvar_t create_poly64(void *solver, bvpoly64_t *p, thvar_t *map)
  *    - must return a theory variable that represents p with variables renamed as
@@ -545,6 +550,7 @@ typedef struct arith_interface_s {
 typedef thvar_t (*create_bv_var_fun_t)(void *solver, uint32_t nbits);
 typedef thvar_t (*create_bv_const_fun_t)(void *solver, bvconst_term_t *c);
 typedef thvar_t (*create_bv64_const_fun_t)(void *solver, bvconst64_term_t *c);
+typedef thvar_t (*create_bv_zero_fun_t)(void *solver, uint32_t nbits);
 typedef thvar_t (*create_bv_poly_fun_t)(void *solver, bvpoly_t *p, thvar_t *map);
 typedef thvar_t (*create_bv64_poly_fun_t)(void *solver, bvpoly64_t *p, thvar_t *map);
 typedef thvar_t (*create_bv_pprod_fun_t)(void *solver, pprod_t *p, thvar_t *map);
@@ -561,6 +567,7 @@ typedef struct bv_interface_s {
   create_bv_var_fun_t create_var;
   create_bv_const_fun_t create_const;
   create_bv64_const_fun_t create_const64;
+  create_bv_zero_fun_t create_zero;
   create_bv_poly_fun_t create_poly;
   create_bv64_poly_fun_t create_poly64;
   create_bv_pprod_fun_t create_pprod;
@@ -646,6 +653,7 @@ struct context_s {
   void *arith_solver;
   void *bv_solver;
   void *fun_solver;
+  void *quant_solver;
 
   // solver internalization interfaces
   arith_interface_t arith;
@@ -674,6 +682,7 @@ struct context_s {
   ivector_t aux_vector;
   int_queue_t queue;
   int_stack_t istack;
+  objstack_t ostack;
 
   // data about shared subterms
   sharing_map_t sharing;
@@ -703,6 +712,9 @@ struct context_s {
   polynomial_t *aux_poly;
   uint32_t aux_poly_size;  // number of monomials in aux_poly
 
+  // buffers for bitvector simplification
+  bvpoly_buffer_t *bvpoly_buffer;
+
   // auxiliary buffers for model construction
   rational_t aux;
   bvconstant_t bv_buffer;
@@ -715,6 +727,9 @@ struct context_s {
 
   // options for the mcsat solver
   mcsat_options_t mcsat_options;
+
+  // flag for enabling adding quant instances
+  bool en_quant;
 };
 
 
@@ -758,20 +773,23 @@ enum {
   FORMULA_NOT_IDL = -14,
   FORMULA_NOT_RDL = -15,
   FORMULA_NOT_LINEAR = -16,
-  TOO_MANY_ARITH_VARS = -17,
-  TOO_MANY_ARITH_ATOMS = -18,
-  ARITHSOLVER_EXCEPTION = -19,
+  DIV_BY_ZERO = -17,
+  TOO_MANY_ARITH_VARS = -18,
+  TOO_MANY_ARITH_ATOMS = -19,
+  ARITHSOLVER_EXCEPTION = -20,
   // bv solver errors
-  BVSOLVER_EXCEPTION = -20,
+  BVSOLVER_EXCEPTION = -21,
   // mcsat errors
-  MCSAT_EXCEPTION_UNSUPPORTED_THEORY = -21
+  MCSAT_EXCEPTION_UNSUPPORTED_THEORY = -22,
+  // new code: added 2021/06/29
+  HIGH_ORDER_FUN_NOT_SUPPORTED = -23,
 };
 
 
 /*
  * NUM_INTERNALIZATION_ERRORS: must be (1 + number of negative codes)
  */
-#define NUM_INTERNALIZATION_ERRORS 22
+#define NUM_INTERNALIZATION_ERRORS 24
 
 
 
