@@ -29,6 +29,7 @@
 #include <string.h>
 
 #include "api/yices_extensions.h"
+#include "api/yices_api_lock_free.h"
 #include "api/yices_globals.h"
 #include "frontend/smt2/attribute_values.h"
 #include "frontend/smt2/smt2_commands.h"
@@ -163,8 +164,6 @@ static void check_integer_term(tstack_t *stack, stack_elem_t *e) {
  */
 
 /*
- * TODO: deal with other SMT2 nonsense
- *
  * 1) mk_implies can take more than two arguments
  *    (=> a b c) is interpreted as (=> a (=> b c))
  *
@@ -175,6 +174,10 @@ static void check_integer_term(tstack_t *stack, stack_elem_t *e) {
  * 3) missing operators: div, mod, abs, divisible
  *    note: div is marked as left-associative!
  *    (div a b c) is interpreted as (div (div a b) c)
+ *
+ * 4) restrictions on bvnand, bvnor, bvxnor
+ *
+ * 5) n-ary division
  */
 
 /*
@@ -216,13 +219,12 @@ static void eval_smt2_mk_bv_rotate_left(tstack_t *stack, stack_elem_t *f, uint32
   index = get_integer(stack, f);
   b = tstack_get_bvlbuffer(stack);
   bvl_set_elem(stack, b, f+1);
-  if (! yices_check_bitshift(b, index)) {
+  if (! yices_check_smt2_rotate(b, &index)) {
     report_yices_error(stack);
   }
   // we known 0 <= index <= bitsize of b
-  if (index < bvlogic_buffer_bitsize(b)) {
-    bvlogic_buffer_rotate_left(b, index);
-  }
+  bvlogic_buffer_rotate_left(b, index);
+
   tstack_pop_frame(stack);
   set_bvlogic_result(stack, b);
 }
@@ -245,13 +247,12 @@ static void eval_smt2_mk_bv_rotate_right(tstack_t *stack, stack_elem_t *f, uint3
   index = get_integer(stack, f);
   b = tstack_get_bvlbuffer(stack);
   bvl_set_elem(stack, b, f+1);
-  if (! yices_check_bitshift(b, index)) {
+  if (! yices_check_smt2_rotate(b, &index)) {
     report_yices_error(stack);
   }
-  // we known 0 <= index <= bitsize of b
-  if (index < bvlogic_buffer_bitsize(b)) {
-    bvlogic_buffer_rotate_right(b, index);
-  }
+  // we known 0 <= index < bitsize of b
+  bvlogic_buffer_rotate_right(b, index);
+
   tstack_pop_frame(stack);
   set_bvlogic_result(stack, b);
 }
@@ -346,6 +347,83 @@ static void eval_smt2_mk_implies(tstack_t *stack, stack_elem_t *f, uint32_t n) {
 }
 
 
+/*
+ * bvnand, bvnor, bvxnor:
+ *
+ * In the default implementation (term_stack2.c), we use the following rules
+ *   (bvnand x_1 ... x_n) = (bvnot (bvand x_1 ... x_n))
+ *   (bvnor x_1 ... x_n)  = (bvnot (bvor x_1 ... x_n))
+ *   (bvxnor x_1 ... x_n) = (bvnot (bvxor x_1 ... x_n))
+ *
+ * In SMT-LIB2, we want to limit them to two arguments only. CVC4 and z3 use
+ * this rule for bvxnor
+ *   (bxnor x_1 x_2 x_3) = (bvxnor (bvxnor x1 x2) x3)
+ * The standard says that bvxnor is not associatived.
+ *
+ * To be safe, we restrict bvnand, bvnor, bvxnor to exactly two arguments here.
+ */
+
+/*
+ * [mk-bv-nand <bv> <bv>]
+ */
+static void check_smt2_mk_bv_nand(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  check_op(stack, MK_BV_NAND);
+  check_size(stack, n == 2);
+}
+
+static void eval_smt2_mk_bv_nand(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  bvlogic_buffer_t *b;
+
+  b = tstack_get_bvlbuffer(stack);
+  bvl_set_elem(stack, b, f);
+  bvand_elem(stack, b, f+1);
+  bvlogic_buffer_not(b);
+  tstack_pop_frame(stack);
+  set_bvlogic_result(stack, b);
+}
+
+
+/*
+ * [mk-bv-nor <bv> <bv>]
+ */
+static void check_smt2_mk_bv_nor(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  check_op(stack, MK_BV_NOR);
+  check_size(stack, n == 2);
+}
+
+static void eval_smt2_mk_bv_nor(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  bvlogic_buffer_t *b;
+
+  b = tstack_get_bvlbuffer(stack);
+  bvl_set_elem(stack, b, f);
+  bvor_elem(stack, b, f+1);
+  bvlogic_buffer_not(b);
+  tstack_pop_frame(stack);
+  set_bvlogic_result(stack, b);
+}
+
+
+/*
+ * [mk-bv-xnor <bv> <bv>]
+ */
+static void check_smt2_mk_bv_xnor(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  check_op(stack, MK_BV_XNOR);
+  check_size(stack, n == 2);
+}
+
+static void eval_smt2_mk_bv_xnor(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  bvlogic_buffer_t *b;
+
+  b = tstack_get_bvlbuffer(stack);
+  bvl_set_elem(stack, b, f);
+  bvxor_elem(stack, b, f+1);
+  bvlogic_buffer_not(b);
+  tstack_pop_frame(stack);
+  set_bvlogic_result(stack, b);
+}
+
+
+
 
 /*
  * CHAINABLE OPERATORS
@@ -660,8 +738,6 @@ static void eval_smt2_abs(tstack_t *stack, stack_elem_t *f, uint32_t n) {
 static void check_smt2_div(tstack_t *stack, stack_elem_t *f, uint32_t n) {
   check_op(stack, SMT2_MK_DIV);
   check_size(stack, n >= 2);
-
-  //  fprintf(stderr, "div\n");
 }
 
 static void eval_smt2_div(tstack_t *stack, stack_elem_t *f, uint32_t n) {
@@ -689,8 +765,6 @@ static void eval_smt2_div(tstack_t *stack, stack_elem_t *f, uint32_t n) {
 static void check_smt2_mod(tstack_t *stack, stack_elem_t *f, uint32_t n) {
   check_op(stack, SMT2_MK_MOD);
   check_size(stack, n == 2);
-
-  //  fprintf(stderr, "mod\n");
 }
 
 static void eval_smt2_mod(tstack_t *stack, stack_elem_t *f, uint32_t n) {
@@ -715,8 +789,6 @@ static void eval_smt2_mod(tstack_t *stack, stack_elem_t *f, uint32_t n) {
 static void check_smt2_divisible(tstack_t *stack, stack_elem_t *f, uint32_t n) {
   check_op(stack, SMT2_MK_DIVISIBLE);
   check_size(stack, n == 2);
-
-  //  fprintf(stderr, "divisible\n");
 }
 
 static void eval_smt2_divisible(tstack_t *stack, stack_elem_t *f, uint32_t n) {
@@ -732,6 +804,107 @@ static void eval_smt2_divisible(tstack_t *stack, stack_elem_t *f, uint32_t n) {
 }
 
 
+/*
+ * n-ary division: [/ x1 x2 ... xn]
+ *
+ * This is interpreted as left-associative:
+ * (/ a b c) is (/ (/ a b) c)
+ *
+ * We rewrite this to (/ a (* b c))
+ */
+static void check_smt2_mk_division(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  check_op(stack, MK_DIVISION);
+  check_size(stack, n >= 2);
+}
+
+/*
+ * Auxiliary function: compute the divisor = product f[0] x ... x f[n-1]
+ * If the divisor is a non-zero rational, it is returned in d and the function
+ * returns NULL_TERM
+ * Otherwise, the function leaves d unchanged and returns a term equal to the divisor
+ */
+static term_t make_divisor_product(tstack_t *stack, stack_elem_t *f, uint32_t n, rational_t *d) {
+  rba_buffer_t *b;
+  mono_t *m;
+  uint32_t i;
+  term_t t;
+
+  assert(n >= 1);
+
+  t = NULL_TERM;
+  if (n == 1) {
+    if (elem_is_nz_constant(f, d)) {
+      assert(q_is_nonzero(d));
+    } else {
+      t = get_term(stack, f);
+    }
+  } else {
+    // compute the product f[0] x ... x f[n-1]
+    b = yices_new_arith_buffer();
+    assert(rba_buffer_is_zero(b));
+
+    // note: this loop may fail and call longjmp
+    // but the buffer b will be deleted on a call to yices_exit
+    // so there's no risk of memory leak.
+    add_elem(stack, b, f);
+    for (i=1; i<n; i++) {
+      mul_elem(stack, b, f+i);
+    }
+
+    if (rba_buffer_is_nonzero(b)) {
+      // b is a non-zero constant
+      m = rba_buffer_get_constant_mono(b);
+      assert(m != NULL);
+      q_set(d, &m->coeff);
+      assert(q_is_nonzero(d));
+    } else {
+      t = arith_buffer_get_term(b);
+    }
+    yices_free_arith_buffer(b);
+  }
+
+  return t;
+}
+
+static void eval_smt2_mk_division(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  rba_buffer_t *b;
+  rational_t divider;
+  term_t t1, t2, t;
+
+  assert(n >= 2);
+
+  q_init(&divider);
+  t2 = make_divisor_product(stack, f+1, n-1, &divider);
+  if (t2 == NULL_TERM) {
+    // division by a constant
+    assert(q_is_nonzero(&divider));
+    if (f->tag == TAG_RATIONAL) {
+      q_div(&f->val.rational, &divider);
+      copy_result_and_pop_frame(stack, f);
+    } else {
+      b = tstack_get_abuffer(stack);
+      add_elem(stack, b, f);
+      rba_buffer_div_const(b, &divider);
+      tstack_pop_frame(stack);
+      set_arith_result(stack, b);
+    }
+  } else {
+    // the divisor is t2
+    t1 = get_term(stack, f);
+    t = _o_yices_division(t1, t2);
+    check_term(stack, t);
+    tstack_pop_frame(stack);
+    set_term_result(stack,  t);
+  }
+
+  /*
+   * It's safe to clear the divider only here.
+   * If the code above raises an exception, divider is still 0/1
+   * and q_clear would do nothing anyway.
+   */
+  q_clear(&divider);
+}
+
 
 /*
  * NEW OPCODES
@@ -842,6 +1015,21 @@ static void eval_smt2_get_unsat_assumptions(tstack_t *stack, stack_elem_t *f, ui
 }
 
 
+/*
+ * [get-unsat-model-interpolant]
+ */
+static void check_smt2_get_unsat_model_interpolant(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  check_op(stack, SMT2_GET_UNSAT_MODEL_INTERPOLANT);
+  check_size(stack, n == 0);
+}
+
+static void eval_smt2_get_unsat_model_interpolant(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  smt2_get_unsat_model_interpolant();
+  tstack_pop_frame(stack);
+  no_result(stack);
+}
+
+
 /*
  * [get-value <term> .... <term>]
  */
@@ -1063,6 +1251,76 @@ static void eval_smt2_check_sat_assuming(tstack_t *stack, stack_elem_t *f, uint3
 }
 
 
+static void check_constant_term(tstack_t *stack, stack_elem_t *e) {
+  term_t t = get_term(stack, e);
+  term_constructor_t kind = yices_term_constructor(t);
+  switch (kind) {
+  case YICES_BOOL_CONSTANT:
+  case YICES_ARITH_CONSTANT:
+  case YICES_BV_CONSTANT:
+    break;
+  default:
+    raise_exception(stack, e, TSTACK_NOT_A_CONSTANT);
+  }
+}
+
+static void check_all_constants_terms(tstack_t *stack, stack_elem_t *e, stack_elem_t *end) {
+  while (e < end) {
+    check_constant_term(stack, e);
+    e ++;
+  }
+}
+
+static void check_variable(tstack_t *stack, stack_elem_t *e) {
+  term_t t = get_term(stack, e);
+  term_constructor_t kind = yices_term_constructor(t);
+  switch (kind) {
+  case YICES_UNINTERPRETED_TERM:
+    break;
+  default:
+    raise_exception(stack, e, TSTACK_NOT_A_VARIABLE);
+  }
+}
+
+static void check_all_variables(tstack_t *stack, stack_elem_t *e, stack_elem_t *end) {
+  while (e < end) {
+    check_variable(stack, e);
+    e ++;
+  }
+}
+
+/*
+ * [check-sat-assuming (<symbol>*) (<const term>*) ]
+ */
+static void check_smt2_check_sat_assuming_model(tstack_t *stack, stack_elem_t *f, uint32_t m) {
+  uint32_t n;
+
+  if (m % 2) {
+    raise_exception(stack, f, TSTACK_VARIABLES_VALUES_NOT_MATCHING);
+  }
+  n = m / 2;
+  check_all_variables(stack, f, f + n);
+  check_all_constants_terms(stack, f + n, f + m);
+}
+
+static void eval_smt2_check_sat_assuming_model(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  uint32_t i, m;
+  term_t* vars_and_values;
+
+  vars_and_values = (term_t*) safe_malloc(n * sizeof(term_t));
+
+  for (i = 0; i < n; ++ i) {
+    vars_and_values[i] = get_term(stack, f + i);
+  }
+
+  m = n / 2;
+  smt2_check_sat_assuming_model(m, vars_and_values, vars_and_values + m);
+
+  tstack_pop_frame(stack);
+  no_result(stack);
+
+  safe_free(vars_and_values);
+}
 
 /*
  * [declare-sort <symbol> <numeral>]
@@ -1133,7 +1391,7 @@ static void eval_smt2_declare_fun(tstack_t *stack, stack_elem_t *f, uint32_t n)
 
   ntypes = n - 1;
   a = get_aux_buffer(stack, ntypes);
-  for (i=0; i<n; i++) {
+  for (i=0; i<ntypes; i++) {
     a[i] = f[i+1].val.type;
   }
 
@@ -1212,9 +1470,11 @@ static void check_smt2_reset_assertions(tstack_t *stack, stack_elem_t *f, uint32
 }
 
 static void eval_smt2_reset_assertions(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  tstack_reset(stack);
+  tstack_reset_buffers(stack);
   smt2_reset_assertions();
-  tstack_pop_frame(stack);
-  no_result(stack);
+  //  tstack_pop_frame(stack);
+  //  no_result(stack);
 }
 
 /*
@@ -1225,10 +1485,13 @@ static void check_smt2_reset_all(tstack_t *stack, stack_elem_t *f, uint32_t n) {
   check_size(stack, n == 0);
 }
 
+// side effect: this resets the stack and free buffers
 static void eval_smt2_reset_all(tstack_t *stack, stack_elem_t *f, uint32_t n) {
+  tstack_reset(stack);
+  tstack_reset_buffers(stack);
   smt2_reset_all();
-  tstack_pop_frame(stack);
-  no_result(stack);
+  //  tstack_pop_frame(stack);
+  //  no_result(stack);
 }
 
 
@@ -1512,14 +1775,14 @@ static const int32_t smt2_val[NUM_SMT2_SYMBOLS] = {
   MK_LT,                 // SMT2_SYM_LT
   MK_GE,                 // SMT2_SYM_GE
   MK_GT,                 // SMT2_SYM_GT
-  SMT2_MK_DIV,           // SMT2_SYM_DIV (integer division not supported)
+  SMT2_MK_DIV,           // SMT2_SYM_DIV (integer division)
   SMT2_MK_MOD,           // SMT2_SYM_MOD
   SMT2_MK_ABS,           // SMT2_SYM_ABS
-  SMT2_MK_TO_REAL,       // SMT2_SYM_TO_REAL (?? could use a NO_OP for that one)
+  SMT2_MK_TO_REAL,       // SMT2_SYM_TO_REAL (NO_OP for that one)
   SMT2_MK_TO_INT,        // SMT2_SYM_TO_INT
   SMT2_MK_IS_INT,        // SMT2_SYM_IS_INT
   SMT2_MK_DIVISIBLE,     // SMT2_SYM_DIVISIBLE
-  MK_BV_CONST,           // SMT2_SYM_BV_CONSTANT ( for _bv<value> size)
+  MK_BV_CONST,           // SMT2_SYM_BV_CONSTANT (for _bv<value> size)
   MK_BV_TYPE,            // SMT2_SYM_BITVEC
   MK_BV_CONCAT,          // SMT2_SYM_CONCAT
   MK_BV_EXTRACT,         // SMT2_SYM_EXTRACT
@@ -2366,6 +2629,9 @@ void init_smt2_tstack(tstack_t *stack) {
   tstack_add_op(stack, MK_GT, false, eval_smt2_mk_gt, check_smt2_mk_gt);
   tstack_add_op(stack, MK_LE, false, eval_smt2_mk_le, check_smt2_mk_le);
   tstack_add_op(stack, MK_LT, false, eval_smt2_mk_lt, check_smt2_mk_lt);
+  tstack_add_op(stack, MK_BV_NAND, false, eval_smt2_mk_bv_nand, check_smt2_mk_bv_nand);
+  tstack_add_op(stack, MK_BV_NOR, false, eval_smt2_mk_bv_nor, check_smt2_mk_bv_nor);
+  tstack_add_op(stack, MK_BV_XNOR, false, eval_smt2_mk_bv_xnor, check_smt2_mk_bv_xnor);
 
   tstack_add_op(stack, SMT2_EXIT, false, eval_smt2_exit, check_smt2_exit);
   tstack_add_op(stack, SMT2_SILENT_EXIT, false, eval_smt2_silent_exit, check_smt2_silent_exit);
@@ -2374,6 +2640,7 @@ void init_smt2_tstack(tstack_t *stack) {
   tstack_add_op(stack, SMT2_GET_PROOF, false, eval_smt2_get_proof, check_smt2_get_proof);
   tstack_add_op(stack, SMT2_GET_UNSAT_CORE, false, eval_smt2_get_unsat_core, check_smt2_get_unsat_core);
   tstack_add_op(stack, SMT2_GET_UNSAT_ASSUMPTIONS, false, eval_smt2_get_unsat_assumptions, check_smt2_get_unsat_assumptions);
+  tstack_add_op(stack, SMT2_GET_UNSAT_MODEL_INTERPOLANT, false, eval_smt2_get_unsat_model_interpolant, check_smt2_get_unsat_model_interpolant);
   tstack_add_op(stack, SMT2_GET_VALUE, false, eval_smt2_get_value, check_smt2_get_value);
   tstack_add_op(stack, SMT2_GET_OPTION, false, eval_smt2_get_option, check_smt2_get_option);
   tstack_add_op(stack, SMT2_GET_INFO, false, eval_smt2_get_info, check_smt2_get_info);
@@ -2385,6 +2652,7 @@ void init_smt2_tstack(tstack_t *stack) {
   tstack_add_op(stack, SMT2_ASSERT, false, eval_smt2_assert, check_smt2_assert);
   tstack_add_op(stack, SMT2_CHECK_SAT, false, eval_smt2_check_sat, check_smt2_check_sat);
   tstack_add_op(stack, SMT2_CHECK_SAT_ASSUMING, false, eval_smt2_check_sat_assuming, check_smt2_check_sat_assuming);
+  tstack_add_op(stack, SMT2_CHECK_SAT_ASSUMING_MODEL, false, eval_smt2_check_sat_assuming_model, check_smt2_check_sat_assuming_model);
   tstack_add_op(stack, SMT2_DECLARE_SORT, false, eval_smt2_declare_sort, check_smt2_declare_sort);
   tstack_add_op(stack, SMT2_DEFINE_SORT, false, eval_smt2_define_sort, check_smt2_define_sort);
   tstack_add_op(stack, SMT2_DECLARE_FUN, false, eval_smt2_declare_fun, check_smt2_declare_fun);
@@ -2416,4 +2684,5 @@ void init_smt2_tstack(tstack_t *stack) {
   tstack_add_op(stack, SMT2_MK_TO_INT, false, eval_smt2_to_int, check_smt2_to_int);
   tstack_add_op(stack, SMT2_MK_IS_INT, false, eval_smt2_is_int, check_smt2_is_int);
   tstack_add_op(stack, SMT2_MK_DIVISIBLE, false, eval_smt2_divisible, check_smt2_divisible);
+  tstack_add_op(stack, MK_DIVISION, false, eval_smt2_mk_division, check_smt2_mk_division);
 }
