@@ -59,6 +59,7 @@
 #include "solvers/floyd_warshall/rdl_floyd_warshall.h"
 #include "solvers/floyd_warshall/rdl_fw_printer.h"
 #include "solvers/funs/fun_solver.h"
+#include "solvers/quant/quant_solver.h"
 #include "solvers/simplex/simplex.h"
 #include "solvers/simplex/simplex_printer.h"
 
@@ -74,9 +75,7 @@
 /*
  * yices_rev is set up at compile time in yices_version.c
  */
-#ifndef NDEBUG
 extern const char * const yices_rev;
-#endif
 
 
 /*
@@ -132,6 +131,7 @@ static const char * const code2error[NUM_INTERNALIZATION_ERRORS] = {
   "not an IDL formula",
   "not an RDL formula",
   "non-linear arithmetic not supported",
+  "division by zero is not supported",
   "too many variables for the arithmetic solver",
   "too many atoms for the arithmetic solver",
   "arithmetic solver exception",
@@ -1051,9 +1051,7 @@ static void print_yices_header(FILE *f) {
   fprintf(f, "GMP %s, Copyright Free Software Foundation, Inc\n", gmp_version);
   fprintf(f, "Build date: %s\n", yices_build_date);
   fprintf(f, "Platform: %s (%s)\n", yices_build_arch, yices_build_mode);
-#ifndef NDEBUG
   fprintf(f, "Revision: %s\n", yices_rev);
-#endif
   fprintf(f, "----\n");
 }
 
@@ -1124,6 +1122,16 @@ static void show_funsolver_stats(fun_solver_stats_t *stat) {
   printf(" extensionality axioms   : %"PRIu32"\n", stat->num_extensionality_axiom);
 }
 
+/*
+ * Quantifier solver statistics
+ */
+static void show_quantsolver_stats(quant_solver_stats_t *stat) {
+  printf("Quantifiers\n");
+  printf(" quantifiers             : %"PRIu32"\n", stat->num_quantifiers);
+  printf(" patterns                : %"PRIu32"\n", stat->num_patterns);
+  printf(" instances               : %"PRIu32"\n", stat->num_instances);
+}
+
 /*
  * Simplex statistics
  */
@@ -1211,6 +1219,7 @@ static void print_results(void) {
   egraph_t *egraph;
   simplex_solver_t *simplex;
   fun_solver_t *fsolver;
+  quant_solver_t *qsolver;
   uint32_t resu;
   double mem_used;
 
@@ -1234,6 +1243,10 @@ static void print_results(void) {
       fsolver = context.fun_solver;
       show_funsolver_stats(&fsolver->stats);
     }
+    if (context_has_quant_solver(&context)) {
+      qsolver = context.quant_solver;
+      show_quantsolver_stats(&qsolver->stats);
+    }
   }
 
   if (context_has_simplex_solver(&context)) {
