@@ -18,11 +18,16 @@
 
 #include "options.h"
 
+#include <stddef.h>
+
 extern void init_mcsat_options(mcsat_options_t *opts) {
   opts->nra_nlsat = false;
   opts->nra_mgcd = false;
   opts->nra_bound = false;
   opts->nra_bound_min = -1;
   opts->nra_bound_max = -1;
+  opts->bv_var_size = -1;
+  opts->var_order = NULL;
+  opts->model_interpolation = false;
 }
 
