@@ -168,7 +168,7 @@ static uint32_t store_ordata(ordata_array_t *a, literal_t *b, uint32_t n) {
   if (n >= MAX_ORDATA_ARRAY_SIZE) {
     out_of_memory();
   }
-  // the sum a->top + n + 1 can't oveflow
+  // the sum a->top + n + 1 can't overflow
   ordata_array_resize(a, a->top + n + 1);
 
   assert(a->top + n + 1 <= a->size);
@@ -877,35 +877,26 @@ static int32_t build_orgate_hobj(orgate_hobj_t *o) {
 }
 
 
-
-/*
- * Global objects for hash consing
- */
-static gate_hobj_t gate_hobj = {
-  { (hobj_hash_t) hash_gate_hobj, (hobj_eq_t) eq_gate_hobj, (hobj_build_t) build_gate_hobj },
-  NULL,
-  { 0, { 0, 0, 0 }},
-};
-
-static orgate_hobj_t orgate_hobj = {
-  { (hobj_hash_t) hash_orgate_hobj, (hobj_eq_t) eq_orgate_hobj, (hobj_build_t) build_orgate_hobj },
-  NULL,
-  0,
-  NULL,
-};
-
-
-
 /*
  * Hash-consing constructors
  */
 static bvar_t get_bgate(bool_vartable_t *table, bgate_t *g) {
+  gate_hobj_t gate_hobj;
+
+  gate_hobj.m.hash = (hobj_hash_t) hash_gate_hobj;
+  gate_hobj.m.eq = (hobj_eq_t) eq_gate_hobj;
+  gate_hobj.m.build = (hobj_build_t) build_gate_hobj;
   gate_hobj.table = table;
   gate_hobj.gate = *g;
   return int_htbl_get_obj(&table->htbl, &gate_hobj.m);
 }
 
 static bvar_t get_bor(bool_vartable_t *table, uint32_t n, literal_t *a) {
+  orgate_hobj_t orgate_hobj;
+
+  orgate_hobj.m.hash = (hobj_hash_t) hash_orgate_hobj;
+  orgate_hobj.m.eq = (hobj_eq_t) eq_orgate_hobj;
+  orgate_hobj.m.build = (hobj_build_t) build_orgate_hobj;
   orgate_hobj.table = table;
   orgate_hobj.n = n;
   orgate_hobj.a = a;
@@ -1557,6 +1548,3 @@ literal_t make_xor(bool_vartable_t *table, uint32_t n, literal_t *a) {
 
   return l ^ sign;
 }
-
-
-
