@@ -186,6 +186,7 @@
 #include "terms/balanced_arith_buffers.h"
 #include "terms/bvarith64_buffers.h"
 #include "terms/bvarith_buffers.h"
+#include "terms/bvpoly_buffers.h"
 #include "terms/pprod_table.h"
 #include "terms/types.h"
 #include "utils/bitvectors.h"
@@ -630,6 +631,13 @@ extern term_t bit_term(term_table_t *table, uint32_t index, term_t bv);
  */
 extern term_t pprod_term(term_table_t *table, pprod_t *r);
 
+/*
+ * Power product from a pp_buffer:
+ * - b must be normalized and non-empty
+ * - each variable in b must be a term defined in table
+ * - the variables must have compatible types
+ */
+extern term_t pprod_term_from_buffer(term_table_t *table, pp_buffer_t *b);
 
 
 
@@ -756,6 +764,16 @@ extern term_t bv64_poly(term_table_t *table, bvarith64_buffer_t *b);
 extern term_t bv_poly(term_table_t *table, bvarith_buffer_t *b);
 
 
+/*
+ * Variant: use a bvpoly_buffer b
+ * - this works only for linear polynomials
+ * - all variables of b must be terms defined in table
+ * - b must be normalized
+ * - b is not modified
+ */
+extern term_t bv_poly_from_buffer(term_table_t *table, bvpoly_buffer_t *b);
+
+
 /*
  * Bitvector formed of arg[0] ... arg[n-1]
  * - n must be positive and no more than YICES_MAX_BVSIZE
@@ -1267,6 +1285,9 @@ static inline bool is_tuple_term(const term_table_t *table, term_t t) {
   return term_type_kind(table, t) == TUPLE_TYPE;
 }
 
+static inline bool is_itype_term(const term_table_t *table, term_t t) {
+  return term_type_kind(table, t) == INSTANCE_TYPE;
+}
 
 // Bitsize of term t
 static inline uint32_t term_bitsize(const term_table_t *table, term_t t) {
@@ -1293,6 +1314,15 @@ static inline bool is_const_term(const term_table_t *table, term_t t) {
   return is_const_kind(term_kind(table, t));
 }
 
+// Check whether t is a bitvector polynomials
+static inline bool is_bvpoly_kind(term_kind_t tag) {
+  return tag == BV64_POLY || tag == BV_POLY;
+}
+
+static inline bool is_bvpoly_term(const term_table_t *table, term_t t) {
+  return is_bvpoly_kind(term_kind(table, t));
+}
+
 
 /*
  * Check the type of a literal.
